"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReqInterceptor = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const common_1 = require("./common");
/**
 * Каждый из перехватов(match) по умолчанию отработает только один раз(onePass = true).
 * Такой подход позволяет контролировать момент передачи данных браузеру по всем перехватам.
 * Эта возможность используется в команде 'waitForAllDataReceived'.
 *
 * В зависимости от настроек задачи, в результате конкретного перехвата браузеру будет отдан:
 *   .) либо ответ сервера
 *       - и при необходимости сохранен в хранилище
 *   .) либо mock
 */
class ReqInterceptor {
    constructor(task, env, onePass = true) {
        this.task = task;
        this.env = env;
        this.onePass = onePass;
        this.interceptions = new Map();
    }
    register(data) {
        data === null || data === void 0 ? void 0 : data.forEach(({ match, type }) => {
            this.interceptions.set(match.toString(), { type });
            this.page.route(match, this.getInterceptor(match));
        });
    }
    getInterceptor(match) {
        this.debug(`listen '${match}'`);
        return (route, req) => __awaiter(this, void 0, void 0, function* () {
            const url = req.url();
            const key = common_1.getUrlPath(url);
            this.debug(` - перехват '${url}'`);
            const { status, body, contentType } = yield this.getResponse(key, req);
            this.saveResponse(key, body, contentType);
            if (!this.page) {
                this.debug(` - ignore interception '${this.task.id}' because the page is closed`);
                return;
            }
            route.fulfill({ status, body, contentType });
            this.debug(` - отдал ответ браузеру`);
            if (this.onePass)
                this.setCompleted(match);
        });
    }
    getResponse(key, req) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.task.mockResponses
                ? yield this.getMock(key)
                : yield this.requestToServer(req);
        });
    }
    getMock(key) {
        return __awaiter(this, void 0, void 0, function* () {
            const { buf, contentType } = this.storage.get(this.task, { type: 'response', key });
            this.debug(` - считан mock, size`, buf.length);
            return {
                status: 200,
                body: buf,
                contentType,
            };
        });
    }
    requestToServer(req) {
        return __awaiter(this, void 0, void 0, function* () {
            this.debug(` - ${req.method()} ${req.url()}`);
            const res = yield node_fetch_1.default(req.url(), {
                method: req.method(),
                headers: req.headers(),
                body: req.postData(),
            });
            const buf = yield res.buffer();
            this.debug(` - ответ сервера, size`, buf.length);
            return {
                status: res.status,
                body: buf,
                contentType: res.headers.get('content-type'),
            };
        });
    }
    saveResponse(key, buf, contentType) {
        if (!this.task.saveResponses)
            return;
        this.storage.set(this.task, { type: 'response', key, contentType }, buf);
    }
    setCompleted(match) {
        const info = this.interceptions.get(match.toString());
        if (info.completed) {
            this.checkAllDataReceived();
            return;
        }
        switch (info.type) {
            case 'single': {
                this.completeInterception(match, info);
                break;
            }
            case 'multiple': { // ЕСЛИ ожидается несколько запросов/ответов ТОГДА от последнего ответа надо подождать какое-то время перед завершением
                if (info.timeout)
                    clearTimeout(info.timeout); // ЕСЛИ таймаут задан ТОГДА он еще не сработал и его надо сбросить, т.к. ориентируемся относительно последнего ответа
                info.timeout = setTimeout(() => this.completeInterception(match, info), 1000);
                break;
            }
            default:
                throw new Error(`Неизвестный тип '${info.type}' перехвата ${match}`);
        }
    }
    completeInterception(match, info) {
        info.completed = true;
        this.page.unroute(match);
        this.debug(`stop listen '${match}'`);
        this.checkAllDataReceived();
    }
    checkAllDataReceived() {
        const incomplete = Array.from(this.interceptions.values()).filter(info => !info.completed);
        if (incomplete.length === 0) {
            this.debug(` - все данные получены`);
            this.task.setAllDataReceived();
        }
    }
    //region Support
    get page() {
        return this.task.page;
    }
    get storage() {
        return this.env.storage;
    }
    debug(...args) {
        return this.env.debug(...args);
    }
}
exports.ReqInterceptor = ReqInterceptor;
