"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Storage = void 0;
const fs_1 = require("fs");
const fs_2 = require("../fs");
const path_1 = require("path");
/**
 * Хранилище реализует структуру вида:
 *
 *  [environment]
 *     |---[screenshot]
 *            |--- task-1.png
 *            |--- task-2.png
 *            |--- ...
 *     |---[response]
 *            |--- [task-1]
 *                    |--- index.json  - индексный файл содержимого директории
 *                    |--- file1
 *                    |--- file2
 *                    |--- ...
 *                    |--- fileN
 *            |--- [task-2]
 *                    |--- index.json
 *                    |--- file1
 *                    |--- file2
 *                    |--- ...
 *                    |--- fileN
 *            |--- ...
 */
class Storage {
    constructor(env) {
        this.env = env;
        this.indexFileName = 'index.json';
        this.initDirs();
        this.clean();
    }
    get(task, meta) {
        this.updatePosition(task, meta);
        const { filePath, contentType } = this.file('get', task, meta);
        const buf = fs_1.readFileSync(filePath);
        this.debug(`read ${meta.type} '${filePath}', size`, buf.length);
        return { buf, contentType };
    }
    set(task, meta, buf) {
        this.updatePosition(task, meta);
        const { filePath, fileName, contentType } = this.file('set', task, meta);
        fs_1.writeFileSync(filePath, buf);
        this.updateIndex(meta, { fileName, contentType });
        this.debug(`write ${meta.type} '${filePath}', size`, buf.length);
    }
    //region Структура хранилища
    initDirs() {
        fs_2.FileCheck.ensureDir(this.dir);
        fs_2.FileCheck.ensureDir(this.environmentDir);
    }
    /**
     * Перед тем, как выполнить операцию в хранилище, надо спозиционироваться в нужное место:
     *  - убедиться, что нужные папки созданы;
     *  - обновить соответствующие переменные.
     */
    updatePosition(task, meta) {
        fs_2.FileCheck.ensureDir(this.getStorageFileTypeDir(meta));
        if (meta.type === 'response') {
            fs_2.FileCheck.ensureDir(this.getTaskDir(task));
        }
    }
    get dir() {
        return this.options.storage.dir;
    }
    get environmentDir() {
        return path_1.join(this.dir, this.env.id);
    }
    getStorageFileTypeDir(meta) {
        return this.storageFileTypeDir = path_1.join(this.environmentDir, meta.type);
    }
    getTaskDir(task) {
        this.taskDir = path_1.join(this.storageFileTypeDir, task.id);
        this.taskIndexFilePath = path_1.join(this.taskDir, this.indexFileName);
        return this.taskDir;
    }
    file(action, task, meta) {
        let fileName, contentType, filePath;
        switch (meta.type) {
            case 'screenshot': {
                const fileType = this.options.screenshot.type;
                fileName = task.id + '.' + fileType;
                contentType = `image/${fileType}`;
                filePath = path_1.join(this.storageFileTypeDir, fileName);
                break;
            }
            case 'response': {
                if (action === 'set') {
                    contentType = meta.contentType;
                    fileName = `${Math.random().toString(36).substr(2, 8)}${getFileExtention(contentType)}`;
                }
                else {
                    const index = this.currentIndex[meta.key];
                    if (index === undefined)
                        throw new Error(`Storage#file: в индексе нет такого ключа '${meta.key}'`);
                    fileName = index.fileName;
                    contentType = index.contentType;
                }
                filePath = path_1.join(this.taskDir, fileName);
                break;
            }
            default:
                throw new Error(`Storage#file: неизвестный тип файла '${meta.type}'`);
        }
        return { fileName, contentType, filePath };
    }
    get currentIndex() {
        return this.getIndex(this.taskIndexFilePath);
    }
    getIndex(path) {
        return fs_2.FileJson.read(path);
    }
    updateIndex(meta, value) {
        if (meta.type !== 'response')
            return;
        const index = this.currentIndex;
        index[meta.key] = value;
        fs_2.FileJson.write(index, this.taskIndexFilePath, true);
    }
    //endregion
    //region Clean
    clean() {
        this.debug('==================================================');
        this.debug(`clean storage`);
        this.cleanDir(this.dir);
    }
    cleanDir(path) {
        if (!fs_1.lstatSync(path).isDirectory())
            return;
        const fileNames = fs_1.readdirSync(path);
        if (fileNames.find(fileName => fileName === this.indexFileName)) { // ЕСЛИ в директории есть индексный файл
            const index = this.getIndex(path_1.join(path, this.indexFileName));
            this.removeFiles(path, index, fileNames);
        }
        else
            fileNames.forEach(file => this.cleanDir(path_1.join(path, file)));
    }
    removeFiles(dir, index, allDirFileNames) {
        this.debug(dir);
        const indexFileNames = Object.values(index).map(value => value.fileName);
        allDirFileNames
            .filter(fileName => !indexFileNames.includes(fileName) && fileName !== this.indexFileName)
            .forEach(fileName => {
            this.debug(` - remove file '${fileName}'`);
            fs_1.unlinkSync(path_1.join(dir, fileName));
        });
    }
    //endregion
    //region Support
    get options() {
        return this.env.options;
    }
    debug(...args) {
        return this.env.debug(...args);
    }
}
exports.Storage = Storage;
const getFileExtention = (contentType) => {
    if (!contentType)
        return '';
    if (contentType.includes('application/json'))
        return '.json';
    if (contentType.includes('image/')) {
        if (contentType.includes('png'))
            return '.png';
        if (contentType.includes('jpg'))
            return '.jpg';
        if (contentType.includes('jpeg'))
            return '.jpeg';
    }
    return '';
};
