/// <reference types="node" />
import { BrowserContextOptions, LaunchOptions, Mouse, Page } from 'playwright';
import { IStoppable, Type } from '@do-while-for-each/common';
import { TPoint } from '@do-while-for-each/math';
import { PixelmatchOptions } from 'pixelmatch';
import { PNG } from 'pngjs';
export interface IAutomationEnvironmentOptions {
    browserType: 'chromium' | 'webkit' | 'firefox';
    browser: LaunchOptions;
    browserContext: BrowserContextOptions;
    screenshot: NonNullable<Parameters<Page['screenshot']>[0]>;
    pixelmatch: PixelmatchOptions;
    storage: {
        variant: Type<IStorage>;
        dir: string;
    };
    baseUrl?: string;
    isDebug?: boolean;
    isLeaveOpen?: {
        env?: boolean;
        pageOnTaskEnd?: boolean;
    };
}
export declare const defaultPixelmatchOptions: PixelmatchOptions;
/**
 * Задача - это одна или несколько команд, которые надо выполнить, чтобы получить результат.
 * Результатом выполнения задачи могут быть¹:
 *    - response от сервера²
 *    - скриншот
 *    - результат сравнения скриншота с ожидаемой картинкой (берется из хранилища)
 *  ¹ в зависимости от самой задачи;
 *  ² для перехвата запросов к серверу, надо знать url, по которому клиент обращается за данными.
 * На следующие вопросы должна отвечать конкретная задача:
 *    - надо ли сохранить скриншот в файл? -> команда 'screenshot'
 *    - надо ли сохранить response в файл? -> поле 'saveResponses'
 *    - надо ли мокать ответ сервера?      -> поле 'mockResponses'
 */
export interface ITask extends Partial<IStoppable> {
    /**
     * Каждая задача должна:
     *   - иметь Id. Одно из основных использований Id - это хранение файлов (скриншот, response)
     *   - отдавать Список команд.
     */
    id: string;
    getScript: () => TCommand[];
    beforeScript?: TCommand[];
    afterScript?: TCommand[];
    page?: Page;
    saveResponses?: boolean;
    mockResponses?: boolean;
    delayForDraw?: number;
    initReqInterceptors?: () => void;
    setAllDataReceived?: () => void;
    allDataReceived?: () => Promise<any>;
    actionsBeforeScreenshot?: () => Promise<void>;
    setScreenshot?: (buf: Buffer) => void;
    screenshot?: () => Promise<Buffer>;
    setCompareScreenshotResult?: (result: IImgCompareResult) => void;
    compareScreenshotResult?: () => Promise<IImgCompareResult>;
    login?: () => Promise<TToken>;
    isActive?: boolean;
    isFinished?: boolean;
}
export interface IImgCompareResult {
    isEqual: boolean;
    diffPixelsCount: number;
    origImg: PNG;
    imgToCompare: PNG;
    diffImg: PNG;
}
export declare type TToken = string;
export interface ILogin {
    username: string;
    password: string;
}
export interface IInterception {
    type: TInterceptionType;
    match: TInterceptionMatch;
}
export declare type TInterceptionType = 'single' | 'multiple';
export declare type TInterceptionMatch = string | RegExp;
export interface IInterceptionInfo {
    type: TInterceptionType;
    timeout?: NodeJS.Timeout;
    completed?: boolean;
}
export declare type TCommand = IScreenshotCommand | ICompareScreenshotCommand | INewPageCommand | IClosePageCommand | IGotoCommand | IWaitCommand | IWaitForAllDataReceivedCommand | IClickElementCommand | IFillCommand | IMouseClickCommand | ILoginCommand;
export interface IScreenshotCommand {
    type: 'screenshot';
    data: {
        save?: boolean;
    };
}
export interface ICompareScreenshotCommand {
    type: 'compareScreenshot';
}
export interface IGotoCommand {
    type: 'goto';
    data: string;
}
export interface INewPageCommand {
    type: 'newPage';
}
export interface IClosePageCommand {
    type: 'closePage';
}
export interface IWaitCommand {
    type: 'wait';
    data?: number;
}
export interface IWaitForAllDataReceivedCommand {
    type: 'waitForAllDataReceived';
}
export interface IClickElementCommand {
    type: 'click';
    data: {
        selector: string;
        options?: Parameters<Page['click']>[1];
        useFullSelector?: boolean;
    };
}
export interface IFillCommand {
    type: 'fill';
    data: {
        selector: string;
        value: string;
        options?: Parameters<Page['fill']>[2];
        useFullSelector?: boolean;
    };
}
export interface IMouseClickCommand {
    type: 'mouseClick';
    data: {
        point: TPoint;
        options?: Parameters<Mouse['click']>[2];
    };
}
export interface ILoginCommand {
    type: 'login';
}
export declare class Command {
    static screenshot(data?: IScreenshotCommand['data']): IScreenshotCommand;
    static get compareScreenshot(): ICompareScreenshotCommand;
    static get newPage(): INewPageCommand;
    static get closePage(): IClosePageCommand;
    static goto(data: IGotoCommand['data']): IGotoCommand;
    static wait(data: IWaitCommand['data']): IWaitCommand;
    static get waitForAllDataReceived(): IWaitForAllDataReceivedCommand;
    static click(data: IClickElementCommand['data']): IClickElementCommand;
    static fill(data: IFillCommand['data']): IFillCommand;
    static mouseClick(data: IMouseClickCommand['data']): IMouseClickCommand;
    static get login(): ILoginCommand;
}
export interface IStorage {
    get(task: ITask, meta: IFileMetadata): IStorageGet;
    set(task: ITask, meta: IFileMetadata, buf: Buffer): void;
    clean(): void;
}
export interface IFileMetadata {
    type: 'screenshot' | 'response';
    key?: string;
    contentType?: string;
}
export interface IStorageGet {
    buf: Buffer;
    contentType: string;
}
export interface IFileInfo {
    fileName: string;
    contentType: string;
    filePath: string;
}
/**
 * Индексный файл - index.json - имеет следующую структуру:
 *{
 *  key: {
 *    fileName:
 *    contentType:
 *  },
 *  ...
 *}
 */
export interface IStorageIndex {
    [key: string]: IStorageIndexValue;
}
export interface IStorageIndexValue {
    fileName: string;
    contentType: string;
}
//# sourceMappingURL=contract.d.ts.map